Console = Object:extend()

function Console:new()
	self.lines = {}
	self.queue = {}
	self.curtime = 0
end


function Console:draw()
	love.graphics.setColor(192,192,192,255)

	love.graphics.push()
	love.graphics.setScissor(800, 0, 500, 600)
	love.graphics.translate(800,0)
	for i, t in ipairs(self.lines) do
		love.graphics.print(t, 10, 10+(i-1)*20)
	end
	love.graphics.setColor(255,255,255,255)
	love.graphics.setScissor()
	love.graphics.pop()
end


function Console:beep(t)
	self:enqueue(function() G.sounds.console_beep:play() end)
end


function Console:error(t)
	self:enqueue(function() G.sounds.console_error:play() end)
end


function Console:enqueue(f, t)
	local rt = (t or 0)
	if #self.queue > 0 then
		rt = rt + self.queue[#self.queue].timeout
	else
		rt = rt + self.curtime
	end
	self.queue[#self.queue+1] = {
		func = f,
		timeout = rt
	}
end


function Console:clear(t)
	self:enqueue(function() self.lines = {} end, t)
end


function Console:update(dt)
	if self.allow_ff then
		if love.keyboard.isDown("return") then
			if #self.queue > 0 then
				dt = self.queue[#self.queue].timeout
			end
		end
	end
	self.curtime = self.curtime + dt

	while #self.queue > 0 and self.queue[1].timeout < self.curtime do
		self.queue[1].func()
		table.remove(self.queue, 1)
	end
end


function Console:barrier(func, dt)
	self:enqueue(func, dt)
end


function Console:print(str, t, cl)
	self:enqueue(function()
		if cl then
			self:doClearLine()
		end
		self:doprint(str)
	end, t)
end


function Console:doprint(str)
	if #self.lines > 28 then
		table.remove(self.lines, 1)
  end

	self.lines[#self.lines+1] = str
end


function Console:doClearLine()
	self.lines[#self.lines] = nil
end
