IngameState = lust.State:extend()
require "renderer"
require "movemesh"
require "player"
require "slime"
require "group"
require "slimehurt"
require "slimecannon"
require "deadguy"


function IngameState:loadLevel(name)
	self.movemesh = MoveMesh("data/level/" .. name .. "_collision.lua")
	self.objects = love.filesystem.load("data/level/" .. name .. "_objects.lua")()
	self.triggers = love.filesystem.load("data/level/" .. name .. "_triggers.lua")()
	self.functions = love.filesystem.load("data/level/" .. name .. "_functions.lua")()
	self.color = G.core.images:load("data/level/" .. name .. "_color.png")
	self.depth = G.core.images:load("data/level/" .. name .. "_depth.png")
end

function IngameState:new(name)
	self.allow_fire = true
	lust.State.new(self)
	self.input_enabled = true

	self:loadLevel(name)

--	self.player = Slime(600, 400, self.movemesh) -- self.objects.Player.pos[1], self.objects.Player.pos[2], self.movemesh)

	self.input = lust.BoundInput()
  self.input:bind_key('up',    function() self:walk( 0,  1) end, function() self:walk( 0, -1, true) end)
  self.input:bind_key('down',  function() self:walk( 0, -1) end, function() self:walk( 0,  1, true) end)
  self.input:bind_key('left',  function() self:walk(-1,  0) end, function() self:walk( 1,  0, true) end)
  self.input:bind_key('right', function() self:walk( 1,  0) end, function() self:walk(-1,  0, true) end)
	self.input:bind_key('fire',  function() self:fire() end, function() end)
  self.input.action_map_func = function(key) return G.keyconfig:mapKeyToAction(key) end

	self.renderer = Renderer()

	self:spawnObjects()

	if self.functions.start then
		self.functions.start(self)
	end

end


function IngameState:fire()
	if self.kill_input then return end
	if self.allow_fire and self.input_enabled then
		self.player:fire()
	end
end


function IngameState:spawnObjects()
	self.entities = Group()

	for n, e in pairs(self.objects) do
		local ent
		if e.type == 'player' then
			ent = Player(e.pos[1], e.pos[2], e.pos[3], self.movemesh)
			self.player = ent
		elseif e.type == 'slime' then
			ent = Slime(e.pos[1], e.pos[2], e.pos[3], self.movemesh)
		elseif e.type == 'slimehurt' then
			ent = SlimeHurt(e.pos[1], e.pos[2], e.pos[3], self.movemesh)
		elseif e.type == 'slimecannon' then
			ent = SlimeCannon(e.pos[1], e.pos[2], e.pos[3], self.movemesh)
	  end
		if ent then
			ent.name = n
			ent.state = self

			self.entities:add(ent)
		else
			print("Unknown: " .. n)
		end
	end
end


function IngameState:update(dt)
	lust.State.update(self, dt)
	self.entities:update(dt)

	for e, t in pairs(self.triggers) do
		if self.player.x >= t.min[1] and self.player.x <= t.max[1]
				and self.player.y >= t.min[2] and self.player.y <= t.max[2] then

			if self.functions[e] then
				self.functions[e](self)
			end
		end
	end

	if self.functions.update then
		self.functions.update(self, dt)
	end
end


function IngameState:draw()
	self.entities:draw(self.renderer)

	self.renderer:drawBackground(self.color, self.depth)
	self.renderer:draw()

	--[[
	for i, e in ipairs(self.entities.children) do
		love.graphics.push()
		local x, y, d = projectPoint(e.x, e.y, e.z)
		love.graphics.translate(x, y)
		if e.collaabb then
			love.graphics.rectangle('line', e.collaabb[1], e.collaabb[2], e.collaabb[3] - e.collaabb[1], e.collaabb[4] - e.collaabb[2])
		end
		love.graphics.scale(1, 0.5)
		love.graphics.circle('line', 0, 0, e.radius or 10)
		love.graphics.pop()
	end
	--]]
--	self.movemesh:draw()
	love.graphics.setFont(G.font)

end
	

function IngameState:allowInput(yesno)
	self.input_enabled = yesno
	self.player.dont_walk = not yesno
end

function IngameState:allowFire(yesno)
	self.allow_fire = yesno
end

function IngameState:killInput(yesno)
	self.kill_input= yesno
end


function IngameState:enter()
	G.core.input:push(self.input)
end


function IngameState:walk(x, y, release)
	if self.kill_input then return end
	self.player:walk(x, y)
end
