require "entity"
require "bigbeam"
require "playerexpl"

Player = Entity:extend()


function quad(f, w, h, i)
	local fw, fh = i:getDimensions()
	local fpr = fw / w
	return G.core.quads:make(((f-1) % fpr) * w, math.floor((f-1) / fpr) * h, w, h, fw, fh)
end


function Player:new(x, y, z, movemesh)
	self.attach = movemesh:locate(x, y, z)
	self.step1 = G.sounds.step1:clone()
	self.step2 = G.sounds.step1:clone()
	self.step2:setPitch(0.80)
	self.step1:setPitch(0.70)

	Entity.new(self, x, y, z)

	self.z = self.attach:height()


	self.frames = {
		idle = {},
		walk = {},
		fire = {}
	}

	local w = 100
	local h = 96

	self.sprites = {}
	for _, anim in ipairs({"walk", "idle", "fire"}) do
		for i = 0, 7 do
			self.sprites[anim .. (i+1)] = G.core.images:load("data/sprites/player_" .. anim .. "_" .. i .. ".png")
		end
	end

	for i = 1, 20 do
		self.frames.walk[i]    = quad(i, w, h, self.sprites.walk1)
		self.frames.idle[i]    = quad(i, w, h, self.sprites.idle1)
		self.frames.idle[41-i] = quad(i, w, h, self.sprites.idle1)
	end

	for i = 1, 10 do
		self.frames.fire[i] = quad(i, w, h, self.sprites.fire1)
	end
	

	self.health = 100
	

	self.anim = "idle"

	self.quads = {}
	local fw, fh = self.sprites["walk1"]:getDimensions()
	local fpr = fw / w
	self.frame = 1
	self.origin = {48, 68}
	--for i = 0, 20 do
	--	self.quads[i+1] = G.core.quads:make((i % fpr) * w, math.floor(i / fpr) * h, w, h, fw, fh)
	--end
	self.dir = 0
	self.speed = {0, 0}
	self.radius =40 
end


function Player:update(dt)
	if self.dead then return end
	Entity.update(self, dt)

	local oldframe = math.floor(self.frame)
	self.frame = self.frame + dt * 20

	if not self.dont_walk then
		if self.speed[1] == 0 then
			if self.speed[2] == 0 then
				self.anim = "idle"
			elseif self.speed[2] == 1 then
				self.dir = 7
			elseif self.speed[2] == -1 then
				self.dir = 3
			end

		elseif self.speed[1] == 1 then
			if self.speed[2] == 0 then
				self.dir = 1
			elseif self.speed[2] == 1 then
				self.dir = 0
			elseif self.speed[2] == -1 then
				self.dir = 2
			end

		elseif self.speed[1] == -1 then
			if self.speed[2] == 0 then
				self.dir = 5
			elseif self.speed[2] == 1 then
				self.dir = 6
			elseif self.speed[2] == -1 then
				self.dir = 4
			end
		end

		if self.speed[1] ~= 0 or self.speed[2] ~= 0 then
			self.anim = "walk"
		end
	end
	

	if self.anim == "walk" and math.floor(self.frame) ~= oldframe then
		if oldframe == 20 then
			self.step1:stop()
			self.step1:play()
		elseif oldframe == 10 then
			self.step2:stop()
			self.step2:play()
		end
	end

	if math.floor(self.frame) > #self.frames[self.anim] then
		self.frame = 1
		if self.next_anim then
			self.anim = self.next_anim
			self.next_anim = nil
		end
		if self.after_anim then
			self.after_anim()
		end
	end

	if not self.dont_walk then
		if self.speed[1] ~= 0 or self.speed[2] ~= 0 then
			local snx, sny = lust.tools.normalize(unpack(self.speed))
			local nx = self.x + snx * 100 * dt
			local ny = self.y + sny * 100 * dt

			local allow = self.attach:moveto(nx, ny)
			if allow then
				self.x = nx
				self.y = ny
				self.z = self.attach:height()

				local c, px, py = self.attach:collision(30)
				if c then
					self.x = self.x + px
					self.y = self.y + py
				end

			end
		end
	end


end


function Player:draw(renderer)


	renderer:add(self.sprites[self.anim .. (self.dir + 1)],
							 self.frames[self.anim][math.floor(self.frame)],
							 self.origin[1], self.origin[2],
							 self.x, self.y, self.z)
end


function Player:walk(x, y)
	self.speed[1] = self.speed[1] + x
	self.speed[2] = self.speed[2] + y

end


local directions = {
	{1, 1},
	{1, 0},
	{1, -1},
	{0, -1},
	{-1, -1},
	{-1, 0},
	{-1, 1},
	{0, 1}
}

local offsets = {
  { 30,  24, 36},
	{ 28,   0, 36},
	{ 15, -24, 36},
	{  2, -24, 36},
	{-15, -24, 36},
	{-28,   0, 36},
  {-30,  24, 36},
	{  2,  24, 36},

}


function Player:hurt(dmg)
	if self.health <= 0 then return end
	self.health = self.health -dmg
	if self.health > 0 then
		G.sounds.player_hit:play()
		G.console:print("Hull damage detected")
		G.console:print("Hull integrity: " .. math.floor(self.health) .. "%")
		self.group:add(PlayerExplosion(self.x, self.y, self.z, self.attach.mesh, 
			100,
			{-50, -50, 40},
			{50, 50, 100}
			))
	else
		G.console:print("Critical system failure:")
		G.console:print("Fusion core containment breached.", 0.02)
		G.console:barrier(function()
			G.sounds.player_dead:play()
			self.group:add(PlayerExplosion(self.x, self.y, self.z, self.attach.mesh, 
				600,
				{-100, -100, 0},
				{100, 100, 200}
				))
			self.dead = true
		end, 0.4)
		local st = G.core.states:top()
		st.tween:to(st.renderer.tint, 3, {[1] = 0, [2] = 0, [3] = 0}):delay(3)
		G.console:print("[R] to restart game", 2)
	end

end


function Player:fire()
	if self.dead then return end

	if not self.dont_walk then
		G.sounds.badass:stop()
		G.sounds.badass:play()
		self.anim = "fire"
		self.frame = 1
		self.next_anim = "idle"
		self.dont_walk = true
		self.after_anim = function()
			self.dont_walk = false
		end

		local dx, dy = unpack(directions[self.dir + 1])
		local ox, oy, oz = unpack(offsets[self.dir + 1])

    for i, e in ipairs(self.group.children) do
      if e:is(Slime) or e:is(SlimeBall) or e:is(SlimeCannon) then
        local dx, dy = e.x - self.x, e.y - self.y
        local l = lust.tools.length(dx, dy)
        if l < 40 or (l < 60 and quantAng(dx, dy) == self.dir) then
          e:hurt(100)
        end
      end
    end


		local beam = BigBeam(self.x + ox, self.y + oy, self.z + oz, dx, dy, self.attach.mesh, self.group)
		self.state.entities:add(beam)
	end
end
