Renderer = Object:extend()

function Renderer:new()
	self.sprites = {}
	self.early_sprites = {}
	self.tint = {255,255,255}

	self.colorCanvas = love.graphics.newCanvas(800, 600)

  self.fgshader = love.graphics.newShader [[
    uniform Image depthMap;
    uniform float depth;
		uniform bool xray;
    vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords) {
      vec4 dm = Texel(depthMap, vec2(screen_coords.x / 800.0, /*1.0 -*/ screen_coords.y / 600.0));
      float d = dm.r * 256.0 + dm.g * 256.0*256.0;
      d = d - 4.0 * 256.0;
      if(depth > d) {
				if(xray) {
					return vec4(1.0, 0.0, 0.0, 0.4 * Texel(texture, texture_coords).z);
				}
        discard;
      }
      return Texel(texture, texture_coords) * color;
    }
  ]]

end


function Renderer:drawBackground(color, depth)
	local oldc = love.graphics.getCanvas()
	love.graphics.setCanvas(self.colorCanvas)
	love.graphics.draw(color)
	love.graphics.setCanvas(oldc)
  self.depthCanvas = depth
  self.fgshader:send("depthMap", depth)
end


function Renderer:drawSprite(s)
--  local px, py, pd = projectPoint(s.wx, s.wy, s.wz)
  self.fgshader:send("depth", s.wz)
	self.fgshader:send("xray", s.xray)
  love.graphics.setBlendMode(s.mode)
  love.graphics.setColor(s.r, s.g, s.b, s.a)
  love.graphics.draw(s.img, s.quad, s.wx, s.wy, 0, s.sx, s.sy, s.ox, s.oy)
end


function Renderer:draw()
	table.sort(self.sprites, function(a, b) return a.wz < b.wz end)

	love.graphics.setCanvas(self.colorCanvas)
	love.graphics.setShader(self.fgshader)

	for i = #self.early_sprites, 1, -1 do
    local s = self.early_sprites[i]
    self:drawSprite(s)
    self.early_sprites[i] = nil
	end

	for i = #self.sprites, 1, -1 do
    local s = self.sprites[i]
    self:drawSprite(s)
    self.sprites[i] = nil
	end
  love.graphics.setColor(255, 255, 255,255)
  love.graphics.setBlendMode("alpha")
	love.graphics.setCanvas()
	love.graphics.setShader()
	love.graphics.setColor(unpack(self.tint))
	love.graphics.draw(self.colorCanvas)
end


function projectPoint(x, y, z)
	return x,
				 600 - y / 2 - math.sin(60 * 3.14159 / 180.0) * z,
				 y * 0.5 - z * math.sin(30 / 180 * 3.14159)

end



function Renderer:add(img, quad, ox, oy, wx, wy, wz, mode, sx, sy, r, g, b, a, early, xray)

  local prx, pry, prd = projectPoint(wx, wy, wz)
  
  local newspr = {
		img = img,
		quad = quad,
		ox = ox,
		oy = oy,
		wx = prx, --wx,
		wy = pry, --wy,
		wz = prd, -- wz,
    mode = mode or "alpha",
    sx = sx or 1,
    sy = sy or 1,
    r = r or 255,
    g = g or 255,
    b = b or 255,
    a = a or 255,
    early = early,
		xray = xray == nil and true or xray
	}

  if not early then
    self.sprites[#self.sprites + 1] = newspr
  else
    self.early_sprites[#self.early_sprites + 1] = newspr
  end
end
