require "entity"

Slime = Entity:extend()

local directions = {
	{1, 1},
	{1, 0},
	{1, -1},
	{0, -1},
	{-1, -1},
	{-1, 0},
	{-1, 1},
	{0, 1}
}

function Slime:new(x, y, z, movemesh)
	self.attach = movemesh:locate(x, y, z)
	self.diesound = G.sounds.slime_die:clone()
	self.movesound = G.sounds.slime_move:clone()
	self.movesound:setPitch( 0.7 + math.random() * 0.6)
	self.attacksound = G.sounds.slime_melee:clone()

  self.health = 100
	self.dont_move = 0

	Entity.new(self, x, y, self.attach:height())

	self.z = self.attach:height()

	self.frames = {
		idle = {},
		walk = {},
    die = {},
    dead = {},
		melee = {}
	}

	self.next_attack = 1

  self.timekeeper = lust.TimeKeeper()

	for i = 1, 20 do
    self.frames.die[i] = i
		self.frames.walk[i] = i
		self.frames.idle[i] = i
		self.frames.idle[41-i] = i
		self.frames.melee[i] = i
	end

	self.frames.dead[1] = self.frames.die[20]


	self.sprites = {}
	for _, anim in ipairs({"die", "walk", "idle", "melee"}) do
		for i = 0, 7 do
			self.sprites[anim .. (i+1)] = G.core.images:load("data/sprites/slime_" .. anim .. "_" .. i .. ".png")
		end
	end

	for i = 1, 8 do
    self.sprites["dead" .. i] = self.sprites["die" .. i]
  end

	self.anim = "idle"

	self.quads = {}
	local w = 70
	local h = 63
	local fw, fh = self.sprites["walk1"]:getDimensions()
	local fpr = fw / w
	self.frame = 1
	self.origin = {35, 40}
	for i = 0, 20 do
		self.quads[i+1] = G.core.quads:make((i % fpr) * w, math.floor(i / fpr) * h, w, h, fw, fh)
	end
	self.dir = 1
	self.speed = {0, 0}

  self.radius = 35 

  self.collaabb = {
    -30, -20,
    30, 20
  }

  self.thinktime = 0
end



function quantAng(x, y)
  local at = math.atan2(y, x)
  local pi = math.pi
  if at >= -pi / 8 and at <= pi / 8 then
    return 2
  elseif at >= pi / 8 and at <= pi * 3 / 8 then
    return 1
  elseif at >= pi * 3 / 8 and at <= pi * 5 / 8 then
    return 8
  elseif at >= pi * 5 / 8 and at <= pi * 7 / 8 then
    return 7
  elseif at >= -pi * 3 / 8 and at <= -pi / 8 then
    return 3
  elseif at >= -pi * 5 / 8 and at <= -pi * 3 / 8 then
    return 4
  elseif at >= -pi * 7 / 8 and at <= -pi * 5 / 8 then
    return 5
  else
    return 6
  end
end


function Slime:update(dt)
	Entity.update(self, dt)

	self.timekeeper:update(dt)

  if self.health > 0 then
		self.next_attack = self.next_attack - dt

    if self.thinktime > 0.25 then
      local p = self.group.named.Player
			if not p then return end

      local dplx, dply = p.x - self.x, p.y - self.y
      local dpl = lust.tools.length(dplx, dply)

      if dpl < 700 and dpl > 50 then

        local px, py = self.attach:cast_ray(dplx, dply)
        local dpp = lust.tools.length(px - self.x, py - self.y)

        if dpl < dpp then
          self.dir = quantAng(dplx, dply)
          self.speed[1], self.speed[2] = lust.tools.normalize(unpack(directions[self.dir]))
					self.movesound:play()
        end
      else
				self.movesound:stop()
        self.speed[1], self.speed[2] = 0, 0
				if dpl < 60 and self.next_attack <= 0 then
					self.timekeeper:after(0.5, function()
						p:hurt(10)
					end)
					self.attacksound:play()
					self.anim = "melee"
					self.frame = 1
					self.next_attack = 1.5
					self.next_anim = "idle"
					self.dont_move = 1
				end
      end

      self.thinktime = 0
    end

    self.thinktime = self.thinktime + dt

    local nx = self.x + self.speed[1] * 50 * dt
    local ny = self.y + self.speed[2] * 50 * dt

    local allow = self.attach:moveto(nx, ny)
    if allow then
      self.x = nx
      self.y = ny
      self.z = self.attach:height()

      local c, px, py = self.attach:collision(30)
      if c then
        self.x = self.x + px
        self.y = self.y + py
      end

    end

		if self.dont_move <= 0 then
			if self.speed[1] ~= 0 or self.speed[2] ~= 0 then
				self.anim = "walk"
			else
				self.anim = "idle"
			end
		else
			self.dont_move = self.dont_move - dt
	  end

  end
  self.frame = self.frame + dt * 20
  if math.floor(self.frame) > #self.frames[self.anim] then
    self.frame = 1
		if self.next_anim then
			self.anim = self.next_anim
			self.next_anim = nil
		end
    if self.health <= 0 then
      self.anim = "dead"
			self.dead = true
    end
  end

end


function Slime:draw(renderer)
	renderer:add(self.sprites[self.anim .. (self.dir)],
							 self.quads[self.frames[self.anim][math.floor(self.frame)]],
							 self.origin[1], self.origin[2],
							 self.x, self.y, self.z, nil, nil, nil, nil, nil, nil, nil, self.health <= 0)
end


function Slime:walk(x, y)
	self.speed[1] = self.speed[1] + x
	self.speed[2] = self.speed[2] + y

end


function Slime:hurt(dmg)
  if self.health > 0 then
    self.health = self.health - dmg

    if self.health <= 0 then
			self.movesound:stop()
			self.diesound:play()
      self.anim = "die"
      self.frame = 1
    end

    local count = math.ceil(dmg / 10)

    for i = 1, count do
      local x, y = love.math.random(-20, 20), love.math.random(-10, 10)
      self.group:add(SlimeHurt(self.x+x, self.y+y, self.z, self.attach.mesh))
    end
  end
end
