require "entity"
require "slimeball"

SlimeCannon = Entity:extend()

function SlimeCannon:new(x, y, z, movemesh)
	self.attach = movemesh:locate(x, y, z)
	self.spinup = G.sounds.cannon_spinup:clone()
	self.throw  = G.sounds.cannon_throw:clone()
	self.diesound = G.sounds.slime_die:clone()
	self.hitsound = G.sounds.slime_hit:clone()
	self.spinup:setPitch(0.8 + 0.4 * math.random())

  self.health = 250
	self.dont_move = 0

	Entity.new(self, x, y, self.attach:height())

	self.z = self.attach:height()

	self.frames = {
	}

	self.next_attack = 2

  self.timekeeper = lust.TimeKeeper()

	for i = 1, 40 do
    self.frames[i] = i
	end

	self.framerate = 1

	self.sprite = G.core.images:load("data/sprites/slime_cannon.png")

	self.quads = {}
	local w = 90
	local h = 75
	local fw, fh = self.sprite:getDimensions()
	local fpr = fw / w
	self.frame = 1
	self.origin = {45, 60}
	for i = 0, 40 do
		self.quads[i+1] = G.core.quads:make((i % fpr) * w, math.floor(i / fpr) * h, w, h, fw, fh)
	end
	self.dir = 1
	self.speed = {0, 0}

  self.radius = 35 

  self.collaabb = {
    -30, -35,
    30,15 
  }

	self.flux = flux.group()
  self.thinktime = 0

end


function SlimeCannon:update(dt)
	Entity.update(self, dt)

	self.timekeeper:update(dt)
	self.flux:update(dt)

  if self.health > 0 then
		self.next_attack = self.next_attack - dt

    if self.thinktime > 0.25 then
      local p = self.group.named.Player
			if not p then return end

      local dplx, dply = p.x - self.x, p.y - self.y
      local dpl = lust.tools.length(dplx, dply)

      if not self.dont_attack and dpl < 1200 then

        local px, py = self.attach:cast_ray(dplx, dply)
        local dpp = lust.tools.length(px - self.x, py - self.y)

        if dpl < dpp then
					self.dont_attack = true
					self.spinup:play()
					self.flux:to(self, 3, {framerate = 6}):
						oncomplete(function()
							self.throw:play()
						  local sb = SlimeBall(self.x, self.y, self.z, self.attach.mesh)
							local dplx, dply = p.x - self.x, p.y - self.y
							sb.vx = 0.8 * dplx
							sb.vy = 0.8 * dply
							self.group:add(sb)
							self.flux:to(self, 1, {framerate = 1})
								:oncomplete(function()
									self.dont_attack = false
								end)
						end)
        end
      else
      end

      self.thinktime = 0
    end

    self.thinktime = self.thinktime + dt

    local nx = self.x + self.speed[1] * 50 * dt
    local ny = self.y + self.speed[2] * 50 * dt

    local allow = self.attach:moveto(nx, ny)
    if allow then
      self.x = nx
      self.y = ny
      self.z = self.attach:height()

      local c, px, py = self.attach:collision(30)
      if c then
        self.x = self.x + px
        self.y = self.y + py
      end

    end

  end
  self.frame = self.frame + dt * 20 * self.framerate
  if math.floor(self.frame) > #self.frames then
    self.frame = 1
  end

end


function SlimeCannon:draw(renderer)
	renderer:add(self.sprite,
							 self.quads[self.frames[math.floor(self.frame)]],
							 self.origin[1], self.origin[2],
							 self.x, self.y, self.z, nil, nil, nil, nil, nil, nil, nil, self.health <= 0)
end


function SlimeCannon:hurt(dmg)
  if self.health > 0 then
		self.hitsound:play()
    self.health = self.health - dmg

    if self.health <= 0 then
			self.spinup:stop()
			self.diesound:play()
--      self.anim = "die"
--      self.frame = 1
      self.dead = true
    end

    local count = math.ceil(dmg / 10)

    for i = 1, count do
      local x, y = love.math.random(-20, 20), love.math.random(-10, 10)
      self.group:add(SlimeHurt(self.x+x, self.y+y, self.z, self.attach.mesh))
    end
  end
end
